@@ -58,18 +58,47 @@ struct _xsltFormat {
     xmlChar		*end;
 };
 
-static char alpha_upper_list[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
-static char alpha_lower_list[] = "abcdefghijklmnopqrstuvwxyz";
-static xsltFormatToken default_token;
+static const char alpha_upper_list[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+static const char alpha_lower_list[] = "abcdefghijklmnopqrstuvwxyz";
+static const xsltFormatToken default_token = {
+    BAD_CAST(DEFAULT_SEPARATOR),
+    DEFAULT_TOKEN,
+    1
+};
 
 /*
- * **** Start temp insert ****
- *
- * The following routine xsltUTF8Charcmp will be replaced with calls to
- * the corresponding libxml routine at a later date (when other
- * inter-library dependencies require it).
+ * Helper functions copied from libxml2
  */
 
+/**
+ * xsltCopyCharMultiByte:
+ * @out:  pointer to an array of xmlChar
+ * @val:  the char value
+ *
+ * append the char value in the array
+ *
+ * Returns the number of xmlChar written
+ */
+static int
+xsltCopyCharMultiByte(xmlChar *out, int val) {
+    if ((out == NULL) || (val < 0)) return(0);
+    if  (val >= 0x80) {
+	xmlChar *savedout = out;
+	int bits;
+	if (val <   0x800) { *out++= (val >>  6) | 0xC0;  bits=  0; }
+	else if (val < 0x10000) { *out++= (val >> 12) | 0xE0;  bits=  6;}
+	else if (val < 0x110000)  { *out++= (val >> 18) | 0xF0;  bits=  12; }
+	else {
+	    return(0);
+	}
+	for ( ; bits >= 0; bits-= 6)
+	    *out++= ((val >> bits) & 0x3F) | 0x80 ;
+	return (out - savedout);
+    }
+    *out = val;
+    return 1;
+}
+
 /**
  * xsltUTF8Charcmp
  * @utf1: pointer to first UTF8 char
@@ -92,7 +121,12 @@ xsltUTF8Charcmp(xmlChar *utf1, xmlChar *utf2) {
     return xmlStrncmp(utf1, utf2, len);
 }
 
-/***** Stop temp insert *****/
+static int
+xsltIsLetterDigit(int val) {
+    return xmlIsBaseCharQ(val) || xmlIsIdeographicQ(val) ||
+           xmlIsDigitQ(val);
+}
+
 /************************************************************************
  *									*
  *			Utility functions				*
@@ -110,7 +144,7 @@ xsltUTF8Charcmp(xmlChar *utf1, xmlChar *utf2) {
 #define IS_DIGIT_ONE(x) xsltIsDigitZero((x)-1)
 
 static int
-xsltIsDigitZero(unsigned int ch)
+xsltIsDigitZero(int ch)
 {
     /*
      * Reference: ftp://ftp.unicode.org/Public/UNIDATA/UnicodeData.txt
@@ -168,7 +202,7 @@ xsltNumberFormatDecimal(xmlBufferPtr buffer,
 		break;
 	    }
 	    pointer -= groupingCharacterLen;
-	    xmlCopyCharMultiByte(pointer, groupingCharacter);
+	    xsltCopyCharMultiByte(pointer, groupingCharacter);
 	}
 
 	val = digit_zero + (int)fmod(number, 10.0);
@@ -177,7 +211,7 @@ xsltNumberFormatDecimal(xmlBufferPtr buffer,
 	        i = -1;
 		break;
 	    }
-	    *(--pointer) = (xmlChar)val;
+	    *(--pointer) = val;
 	}
 	else {
 	/*
@@ -186,7 +220,7 @@ xsltNumberFormatDecimal(xmlBufferPtr buffer,
 	 * it is.  So, we generate it into the buffer temp_char, then
 	 * copy from there into temp_string.
 	 */
-	    len = xmlCopyCharMultiByte(temp_char, val);
+	    len = xsltCopyCharMultiByte(temp_char, val);
 	    if ( (pointer - len) < temp_string ) {
 	        i = -1;
 		break;
@@ -212,7 +246,7 @@ xsltNumberFormatAlpha(xsltNumberDataPtr data,
     char temp_string[sizeof(double) * CHAR_BIT * sizeof(xmlChar) + 1];
     char *pointer;
     int i;
-    char *alpha_list;
+    const char *alpha_list;
     double alpha_size = (double)(sizeof(alpha_upper_list) - 1);
 
     /*
@@ -334,11 +368,6 @@ xsltNumberFormatTokenize(const xmlChar *format,
     int val;
     int len;
 
-    default_token.token = DEFAULT_TOKEN;
-    default_token.width = 1;
-    default_token.separator = BAD_CAST(DEFAULT_SEPARATOR);
-
-
     tokens->start = NULL;
     tokens->tokens[0].separator = NULL;
     tokens->end = NULL;
@@ -347,8 +376,7 @@ xsltNumberFormatTokenize(const xmlChar *format,
      * Insert initial non-alphanumeric token.
      * There is always such a token in the list, even if NULL
      */
-    while (! (IS_LETTER(val=xmlStringCurrentChar(NULL, format+ix, &len)) ||
-	      IS_DIGIT(val)) ) {
+    while (!xsltIsLetterDigit(val = xsltGetUTF8CharZ(format+ix, &len))) {
 	if (format[ix] == 0)		/* if end of format string */
 	    break; /* while */
 	ix += len;
@@ -371,19 +399,19 @@ xsltNumberFormatTokenize(const xmlChar *format,
 	    tokens->end = NULL;
 	}
 
-	val = xmlStringCurrentChar(NULL, format+ix, &len);
+	val = xsltGetUTF8CharZ(format+ix, &len);
 	if (IS_DIGIT_ONE(val) ||
 		 IS_DIGIT_ZERO(val)) {
 	    tokens->tokens[tokens->nTokens].width = 1;
 	    while (IS_DIGIT_ZERO(val)) {
 		tokens->tokens[tokens->nTokens].width++;
 		ix += len;
-		val = xmlStringCurrentChar(NULL, format+ix, &len);
+		val = xsltGetUTF8CharZ(format+ix, &len);
 	    }
 	    if (IS_DIGIT_ONE(val)) {
 		tokens->tokens[tokens->nTokens].token = val - 1;
 		ix += len;
-		val = xmlStringCurrentChar(NULL, format+ix, &len);
+		val = xsltGetUTF8CharZ(format+ix, &len);
 	    } else {
                 tokens->tokens[tokens->nTokens].token = '0';
                 tokens->tokens[tokens->nTokens].width = 1;
@@ -394,7 +422,7 @@ xsltNumberFormatTokenize(const xmlChar *format,
 		    (val == 'i') ) {
 	    tokens->tokens[tokens->nTokens].token = val;
 	    ix += len;
-	    val = xmlStringCurrentChar(NULL, format+ix, &len);
+	    val = xsltGetUTF8CharZ(format+ix, &len);
 	} else {
 	    /* XSLT section 7.7
 	     * "Any other format token indicates a numbering sequence
@@ -414,20 +442,20 @@ xsltNumberFormatTokenize(const xmlChar *format,
 	 * to correspond to the Letter and Digit classes from XML (and
 	 * one wonders why XSLT doesn't refer to these instead).
 	 */
-	while (IS_LETTER(val) || IS_DIGIT(val)) {
+	while (xsltIsLetterDigit(val)) {
 	    ix += len;
-	    val = xmlStringCurrentChar(NULL, format+ix, &len);
+	    val = xsltGetUTF8CharZ(format+ix, &len);
 	}
 
 	/*
 	 * Insert temporary non-alphanumeric final tooken.
 	 */
 	j = ix;
-	while (! (IS_LETTER(val) || IS_DIGIT(val))) {
+	while (!xsltIsLetterDigit(val)) {
 	    if (val == 0)
 		break; /* while */
 	    ix += len;
-	    val = xmlStringCurrentChar(NULL, format+ix, &len);
+	    val = xsltGetUTF8CharZ(format+ix, &len);
 	}
 	if (ix > j)
 	    tokens->end = xmlStrndup(&format[j], ix - j);
@@ -443,7 +471,7 @@ xsltNumberFormatInsertNumbers(xsltNumberDataPtr data,
 {
     int i = 0;
     double number;
-    xsltFormatTokenPtr token;
+    const xsltFormatToken *token;
 
     /*
      * Handle initial non-alphanumeric token
@@ -651,51 +679,48 @@ xsltNumberFormatGetMultipleLevel(xsltTransformContextPtr context,
 {
     int amount = 0;
     int cnt;
-    xmlNodePtr oldCtxtNode;
     xmlNodePtr ancestor;
     xmlNodePtr preceding;
-    xmlXPathParserContextPtr parser;
 
-    oldCtxtNode = context->xpathCtxt->node;
-    parser = xmlXPathNewParserContext(NULL, context->xpathCtxt);
-    if (parser) {
-	/* ancestor-or-self::*[count] */
-	ancestor = node;
-	while ((ancestor != NULL) && (ancestor->type != XML_DOCUMENT_NODE)) {
-	    if ((fromPat != NULL) &&
-		xsltTestCompMatchList(context, ancestor, fromPat))
-		break; /* for */
+    /* ancestor-or-self::*[count] */
+    ancestor = node;
 
-            /*
-             * The xmlXPathNext* iterators require that the context node is
-             * set to the start node. Calls to xsltTestCompMatch* may also
-             * leave the context node in an undefined state, so make sure
-             * that the context node is reset before each iterator invocation.
-             */
+    while ((ancestor != NULL) && (ancestor->type != XML_DOCUMENT_NODE)) {
+        if ((fromPat != NULL) &&
+            xsltTestCompMatchList(context, ancestor, fromPat))
+            break;
 
-	    if (xsltTestCompMatchCount(context, ancestor, countPat, node)) {
-		/* count(preceding-sibling::*) */
-		cnt = 1;
-                context->xpathCtxt->node = ancestor;
-                preceding = xmlXPathNextPrecedingSibling(parser, ancestor);
-                while (preceding != NULL) {
-	            if (xsltTestCompMatchCount(context, preceding, countPat,
-                                               node))
-			cnt++;
-                    context->xpathCtxt->node = ancestor;
-                    preceding =
-                        xmlXPathNextPrecedingSibling(parser, preceding);
-		}
-		array[amount++] = (double)cnt;
-		if (amount >= max)
-		    break; /* for */
-	    }
-            context->xpathCtxt->node = node;
-            ancestor = xmlXPathNextAncestor(parser, ancestor);
-	}
-	xmlXPathFreeParserContext(parser);
+        if (xsltTestCompMatchCount(context, ancestor, countPat, node)) {
+            /* count(preceding-sibling::*) */
+            cnt = 1;
+            if (ancestor->type != XML_NAMESPACE_DECL)
+                preceding = ancestor->prev;
+            else
+                preceding = NULL;
+            while (preceding != NULL) {
+                if (xsltTestCompMatchCount(context, preceding, countPat,
+                                           node))
+                    cnt++;
+                preceding = preceding->prev;
+            }
+            array[amount++] = (double)cnt;
+            if (amount >= max)
+                break;
+        }
+
+        if ((ancestor != NULL) && (ancestor->type == XML_NAMESPACE_DECL)) {
+            xmlNsPtr ns = (xmlNsPtr) ancestor;
+
+            if ((ns->next != NULL) &&
+                (ns->next->type != XML_NAMESPACE_DECL))
+                ancestor = (xmlNodePtr) ns->next;
+            else
+                ancestor = NULL;
+        } else {
+            ancestor = ancestor->parent;
+        }
     }
-    context->xpathCtxt->node = oldCtxtNode;
+
     return amount;
 }
 
@@ -708,9 +733,12 @@ xsltNumberFormatGetValue(xmlXPathContextPtr context,
     int amount = 0;
     xmlBufferPtr pattern;
     xmlXPathObjectPtr obj;
+    xmlNodePtr oldNode;
 
     pattern = xmlBufferCreate();
     if (pattern != NULL) {
+        oldNode = context->node;
+
 	xmlBufferCCat(pattern, "number(");
 	xmlBufferCat(pattern, value);
 	xmlBufferCCat(pattern, ")");
@@ -723,6 +751,8 @@ xsltNumberFormatGetValue(xmlXPathContextPtr context,
 	    xmlXPathFreeObject(obj);
 	}
 	xmlBufferFree(pattern);
+
+        context->node = oldNode;
     }
     return amount;
 }
@@ -959,6 +989,7 @@ xsltFormatNumberConversion(xsltDecimalFormatPtr self,
     xmlChar *the_format, *prefix = NULL, *suffix = NULL;
     xmlChar *nprefix, *nsuffix = NULL;
     int	    prefix_length, suffix_length = 0, nprefix_length, nsuffix_length;
+    int     exp10;
     double  scale;
     int	    j, len = 0;
     int     self_grouping_len;
@@ -979,32 +1010,12 @@ xsltFormatNumberConversion(xsltDecimalFormatPtr self,
 		"Invalid format (0-length)\n");
     }
     *result = NULL;
-    switch (xmlXPathIsInf(number)) {
-	case -1:
-	    if (self->minusSign == NULL)
-		*result = xmlStrdup(BAD_CAST "-");
-	    else
-		*result = xmlStrdup(self->minusSign);
-	    /* Intentional fall-through */
-	case 1:
-	    if ((self == NULL) || (self->infinity == NULL))
-		*result = xmlStrcat(*result, BAD_CAST "Infinity");
-	    else
-		*result = xmlStrcat(*result, self->infinity);
-	    return(status);
-	default:
-	    if (xmlXPathIsNaN(number)) {
-		if ((self == NULL) || (self->noNumber == NULL))
-		    *result = xmlStrdup(BAD_CAST "NaN");
-		else
-		    *result = xmlStrdup(self->noNumber);
-		return(status);
-	    }
-    }
-
-    buffer = xmlBufferCreate();
-    if (buffer == NULL) {
-	return XPATH_MEMORY_ERROR;
+    if (xmlXPathIsNaN(number)) {
+        if ((self == NULL) || (self->noNumber == NULL))
+            *result = xmlStrdup(BAD_CAST "NaN");
+        else
+            *result = xmlStrdup(self->noNumber);
+        return(status);
     }
 
     format_info.integer_hash = 0;
@@ -1277,6 +1288,30 @@ OUTPUT_NUMBER:
 	format_info.add_decimal = TRUE;
     }
 
+    /* Apply multiplier */
+    number *= (double)format_info.multiplier;
+    switch (xmlXPathIsInf(number)) {
+	case -1:
+	    if (self->minusSign == NULL)
+		*result = xmlStrdup(BAD_CAST "-");
+	    else
+		*result = xmlStrdup(self->minusSign);
+	    /* Intentional fall-through */
+	case 1:
+	    if ((self == NULL) || (self->infinity == NULL))
+		*result = xmlStrcat(*result, BAD_CAST "Infinity");
+	    else
+		*result = xmlStrcat(*result, self->infinity);
+	    return(status);
+	default:
+            break;
+    }
+
+    buffer = xmlBufferCreate();
+    if (buffer == NULL) {
+	return XPATH_MEMORY_ERROR;
+    }
+
     /* Ready to output our number.  First see if "default sign" is required */
     if (default_sign != 0)
 	xmlBufferAdd(buffer, self->minusSign, xmlUTF8Strsize(self->minusSign, 1));
@@ -1291,10 +1326,24 @@ OUTPUT_NUMBER:
         j += len;
     }
 
+    /* Round to n digits */
+    number = fabs(number);
+    exp10 = format_info.frac_digits + format_info.frac_hash;
+    /* DBL_MAX_10_EXP should be 308 on IEEE platforms. */
+    if (exp10 > DBL_MAX_10_EXP) {
+        if (format_info.frac_digits > DBL_MAX_10_EXP) {
+            format_info.frac_digits = DBL_MAX_10_EXP;
+            format_info.frac_hash = 0;
+        } else {
+            format_info.frac_hash = DBL_MAX_10_EXP - format_info.frac_digits;
+        }
+        exp10 = DBL_MAX_10_EXP;
+    }
+    scale = pow(10.0, (double) exp10);
+    number += .5 / scale;
+    number -= fmod(number, 1 / scale);
+
     /* Next do the integer part of the number */
-    number = fabs(number) * (double)format_info.multiplier;
-    scale = pow(10.0, (double)(format_info.frac_digits + format_info.frac_hash));
-    number = floor((scale * number + 0.5)) / scale;
     if ((self->grouping != NULL) &&
         (self->grouping[0] != 0)) {
         int gchar;
