@@ -307,6 +307,7 @@ xsltFuzzXslt(const char *data, size_t size) {
     sheet = xsltNewStylesheet();
     if (sheet == NULL)
         goto exit;
+    sheet->opLimit = 10000;
     sheet->xpathCtxt->opLimit = 100000;
     sheet->xpathCtxt->opCount = 0;
     if (xsltParseStylesheetUser(sheet, xsltDoc) != 0)
@@ -714,7 +715,7 @@ xmlFuzzSecondaryEntity(size_t *size) {
 xmlParserInputPtr
 xmlFuzzEntityLoader(const char *URL, const char *ID ATTRIBUTE_UNUSED,
                     xmlParserCtxtPtr ctxt) {
-    xmlParserInputPtr input;
+    xmlParserInputBufferPtr buf;
     xmlFuzzEntityInfo *entity;
 
     if (URL == NULL)
@@ -723,18 +724,10 @@ xmlFuzzEntityLoader(const char *URL, const char *ID ATTRIBUTE_UNUSED,
     if (entity == NULL)
         return(NULL);
 
-    input = xmlNewInputStream(ctxt);
-    if (input == NULL)
+    buf = xmlParserInputBufferCreateMem(entity->data, entity->size,
+                                        XML_CHAR_ENCODING_NONE);
+    if (buf == NULL)
         return(NULL);
-    input->filename = (char *) xmlCharStrdup(URL);
-    input->buf = xmlParserInputBufferCreateMem(entity->data, entity->size,
-                                               XML_CHAR_ENCODING_NONE);
-    if (input->buf == NULL) {
-        xmlFreeInputStream(input);
-        return(NULL);
-    }
-    input->base = input->cur = xmlBufContent(input->buf->buffer);
-    input->end = input->base + entity->size;
 
-    return input;
+    return(xmlNewIOInputStream(ctxt, buf, XML_CHAR_ENCODING_NONE));
 }
